﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Linq;
using Moq;
using System.Data.Entity;
using ELTE.TravelAgency.Models;
using System.Collections.Generic;

namespace ELTE.TravelAgency.Tests
{
    [TestClass]
    public class TravelServiceTest
    {
        private IQueryable<Rent> _data;
        private Mock<DbSet<Rent>> _rentMock;
        private Mock<ITravelAgencyEntities> _entityMock;

        [TestInitialize]
        public void Initialize()
        {
            // a foglalásokt egyenként adjuk meg
            _data = new List<Rent>
            {
                // három foglalás a keresett apartmanra
                new Rent { ApartmentId = 0, StartDate = new DateTime(2016, 06, 06), EndDate = new DateTime(2016, 06, 08) },
                new Rent { ApartmentId = 0, StartDate = new DateTime(2016, 06, 11), EndDate = new DateTime(2016, 06, 12) },
                new Rent { ApartmentId = 0, StartDate = new DateTime(2016, 06, 20), EndDate = new DateTime(2016, 07, 10) },

                // két foglalás másik apartmanokra (nem befolyásolhatják az eredményt)
                new Rent { ApartmentId = 1, StartDate = new DateTime(2016, 06, 01), EndDate = new DateTime(2016, 06, 30) },
                new Rent { ApartmentId = 2, StartDate = new DateTime(2016, 06, 10), EndDate = new DateTime(2016, 06, 11) }
            }.AsQueryable();

            // a foglalások adatbázis táblaként fogjuk visszaadni
            _rentMock = new Mock<DbSet<Rent>>(MockBehavior.Strict);
            _rentMock.As<IQueryable<Rent>>().Setup(r => r.ElementType).Returns(_data.ElementType); // aminek konfigráljuk pár alap tulajdonságát
            _rentMock.As<IQueryable<Rent>>().Setup(r => r.Expression).Returns(_data.Expression);
            _rentMock.As<IQueryable<Rent>>().Setup(r => r.Provider).Returns(_data.Provider);
            _rentMock.As<IQueryable<Rent>>().Setup(r => r.GetEnumerator()).Returns(_data.GetEnumerator()); // majd megadjuk, hogy a listát sorolja fel

            // az entitásmodellt is szimuláljuk
            _entityMock = new Mock<ITravelAgencyEntities>(MockBehavior.Strict);
            _entityMock.Setup<DbSet<Rent>>(entity => entity.Rent).Returns(_rentMock.Object); // és benne a foglalások tábláját

        }

        [TestMethod]
        public void TravelServiceGetRentDatesTest()
        {
            // vezérlő példányosítása
            TravelService service = new TravelService(_entityMock.Object);

            // ellenőrzés januárra
            DateTime[] result = service.GetRentDates(0, 2016, 01).ToArray();
            Assert.AreEqual(0, result.Length);

            // ellenőrzések júniusra
            result = service.GetRentDates(0, 2016, 06).ToArray();

            foreach (DateTime date in _data.Where(rent => rent.ApartmentId == 0).Select(rent => rent.StartDate))
                Assert.IsFalse(result.Contains(date));
            foreach (DateTime date in _data.Where(rent => rent.ApartmentId == 0).Select(rent => rent.EndDate))
                Assert.IsTrue(result.Contains(date));

            // ellenőrzések szeptemberre
            result = service.GetRentDates(0, 2016, 09).ToArray();
            Assert.AreEqual(30, result.Length);

            for (Int32 i = 1; i < result.Length; i++)
            {
                Assert.AreEqual(result[i] - result[i - 1], TimeSpan.FromDays(1));
            }
        }
    }
}
